{
  inputs = {
    quasar-wayland = {
      url = gitlab:jens/quasar-wayland?host=git.c3pb.de;
      inputs.nixpkgs.follows = "nixpkgs";
    };
  };

  outputs = { self, nixpkgs, quasar-wayland }:
  let
    lib = nixpkgs.lib;
    systems = lib.platforms.unix;
    forAllSystems = lib.genAttrs systems;
  in {
    packages = forAllSystems (system:
    let pkgs = import nixpkgs { inherit system; overlays = [
        self.overlay
        quasar-wayland.overlay
        quasar-wayland.overlays.quasar
      ]; };
    in {
      inherit (pkgs.haskell.packages.ghc922) glest;
    }
    );

    overlay = self: super: {
      haskell = super.haskell // {
        packageOverrides = hself: hsuper: super.haskell.packageOverrides hself hsuper // {
          glest = import ./. { pkgs = self; haskellPackages = hself; };
        };
      };
    };

    overlays = {
      quasar = quasar-wayland.overlays.quasar;
      quasar-wayland = quasar-wayland.overlay;
    };

    defaultPackage = forAllSystems (system: self.packages.${system}.glest);

    devShell = forAllSystems (system:
      let
        pkgs = nixpkgs.legacyPackages.${system};
      in pkgs.mkShell {
        inputsFrom = [ self.packages.${system}.glest.env ];
        packages = [
          pkgs.cabal-install
          pkgs.zsh
          pkgs.entr
          pkgs.ghcid
          pkgs.haskell-language-server
        ];
      }
    );
  };
}
